"""
Erweitere die Klassen aus Std-Statement um GUI und PY-Codeausgabe.

Es werden exemplarisch verschiedene Mglichkeiten der Klassenerweiterung genutzt:
a) Direkte Vererbung, b) Mixin-Klasse
Erstellt: 28.02.02 inli
Gendert: 15.04.02 inli
"""

import sys
from StdStatements import *
from qt import *
try:
  from qtcanvas import *
except ImportError:
  pass

# Konstanten
NL = "\n"

def boxtext(painter,x,y,w,h,text,attr=""):
  altfont=painter.font()
  altfontsize=altfont.pixelSize()
  font=QFont()
  size=min(altfontsize,h-1,w/64)
  font.setPixelSize(size)
  
  flag=32
  if attr=="center": flag=flag+4
  if attr=="right": flag=flag+2

  painter.setFont(font)
  painter.drawText(x+w/64,y,w*62/64,h,flag,text)
  
  altfont.setPixelSize(altfontsize)
  painter.setFont(altfont)
  #painter.drawText(x,y,w,h,0,text)
  

class Statement(StdStatement):
    """
    Ein leeres Statement, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 
        if self.body==None:
          items.append(QCanvasRectangle ( x,y,w,h, canvas ) )    
          text=QCanvasText("STATEMENT",canvas)
          text.move(x+w/4,y+h/8)
          items.append(text)
          if w>80 and h>16:
            text2=QCanvasText(_("Klicken = Typ zuweisen!"),canvas)
            text2.move(x+4,y+h/3)
            items.append(text2)
        else:
          items.extend(self.body.out(canvas,x,y,w,h))
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        if self.body==None:
          painter.drawRect( x,y,w,h )    
          boxtext(painter,x,y,w,h,"STATEMENT")

          if w>80 and h>16:
            boxtext(painter,x,y+h/1,w,h/1,_("Klicken = Typ zuweisen!"))
        else:
          self.body.draw(painter,x,y,w,h)

    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        
        if self.body==None:
          r="  "*level + "pass #no Statement" + NL
          liste.append((r,self))
        else:
          self.body.make_Pycode(level,liste)

class Command_mit_out(StdCommand):
    """
    Ein Beispiel fr Mixin-Klassen, erweitert StdCommand um out-Methode
    """

    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 
        items.append(QCanvasRectangle ( x,y,w,h, canvas ) )    

        font=QFont()
        font.setPointSize(6)
        
        t=self.text
        
        if h>4:     
          text2=QCanvasText(t,canvas)
          
          if text2.boundingRect().width()>w or h<16:
            text2.setFont(font)        
          
          if text2.boundingRect().height()>h:
            if t.count("\n")>0:
              t=t[:t.index("\n")]+" #..."
              text2=QCanvasText(t,canvas)
               
          text2.move(x+4,y+2)
          items.append(text2)
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        painter.drawRect( x,y,w,h )    
        boxtext(painter,x,y,w,h,self.text)

class Command_mit_code(StdCommand):        
    """
    Ein Beispiel fr Mixin-Klassen, erweitert StdCommand um PyCode-Methode
    """
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """

        t=self.text.replace(NL,NL + "  "*level)
        r="  "*level + t
        liste.append((r,self))

class Command(Command_mit_out, Command_mit_code):
    """
    Ein Beispiel fr Mixin-Klassen.
    mixin: kombiniert die beiden obigen Sub-Klassen
    """

    pass
   
###

class Ausgabe(StdAusgabe):
    """
    Ein Ausgabe-Statement, erweitert um canvas-Ausgabe und Codeerstellung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 
        items.append(QCanvasRectangle ( x,y,w,h, canvas ) )    
        items.append(QCanvasRectangle ( x+2,y+2,2,h-4, canvas ) )    
       
        text2=QCanvasText(self.text,canvas)
        text2.move(x+5,y+h/4)
        items.append(text2)
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        painter.drawRect( x,y,w,h )    
        painter.drawRect( x+2,y+2,1+w/64,h-4 )    
        boxtext(painter,x+1+w/64,y,w-5,h,self.text)

    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        t="print "+self.text.replace(NL,NL + "  "*level)
        r="  "*level + t + "" # + NL  
        liste.append((r,self))


###
class Eingabe(StdEingabe):
    """
    Ein Eingabe-Statement, erweitert um canvas-Ausgabe und Codeerstellung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 
        items.append(QCanvasRectangle ( x,y,w,h, canvas ) )    
        items.append(QCanvasRectangle ( x+w-4,y+2,2,h-4, canvas ) )    
       
        if self.istext:
          text2=QCanvasText(_("Text? ")+self.name,canvas)
        else:
          text2=QCanvasText(_("Wert? ")+self.name,canvas)

        text2.move(x+4,y+h/4)
        items.append(text2)
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        painter.drawRect( x,y,w,h )    
        painter.drawRect( x+w-w/64-1,y+2,w/64+1,h-4 )    

        if self.istext:
          text2=_("Text? ")+self.name
        else:
          text2=_("Wert? ")+self.name
        boxtext(painter,x+4,y,w-4,h,text2)
      
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        if self.istext:
          t=self.name+' = raw_input("'+self.text+'")'
        else:
          t=self.name+' = input("'+self.text+'")'

        r="  "*level + t + ""
        liste.append((r,self))


###


class Exitcommand(StdExitcommand):
    """
    Ein Exit-Statement, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 
        items.append(QCanvasRectangle ( x,y,w,h, canvas ) )    

        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( x+w/8,y,x,y+h/2)
        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( x,y+h/2,x+w/8,y+h)
        
        text2=QCanvasText(self.text,canvas)
        text2.move(x+w/8,y+h/4)
        items.append(text2)
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        painter.drawRect( x,y,w,h )    
        painter.drawLine( x+w/8,y,x,y+h/2 )
        painter.drawLine( x+w/8,y+h,x,y+h/2 )
        boxtext(painter,x+w/8,y,w*7/8,h,self.text)
        
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        t=self.text.replace(NL,NL + "  "*level)
        r="  "*level + t + "" + NL  
        liste.append((r,self))


###

class Function(StdFunction):
    """
    Ein Function-Statement, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 

        text2=QCanvasText("Function "+self.name+"("+self.text+")",canvas)
        text2.move(x+w/8,y+2)
        text2.setZ(10)
        items.append(text2)

        pen=QPen(QColor(90,255,90))
        brush=QBrush(QColor(100,255,100))
        #items.append(QCanvasRectangle ( x,y,w,18, canvas ) )    

        if self.is_hidden():
          items.append(QCanvasRectangle ( x,y,w,h-2, canvas ) )
          items[-1].setPen(pen)
          items[-1].setBrush(brush)
        else:
          items.append(QCanvasRectangle ( x,y,w,18, canvas ) )
          items[-1].setPen(pen)
          items[-1].setBrush(brush)

          items.append(QCanvasRectangle ( x,y+18,w,h-18, canvas ) )    
          items[-1].setPen(pen)                 
          items.extend(self.body.out(canvas,x+2,y+20,w-4,h-22))

        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        text="Funktion "+self.name+"("+self.text+")"
        
        painter.drawRect( x,y,w,h )    
        if self.is_hidden():
          painter.fillRect( x,y,w,h-2, QBrush(QColor(100,255,100)) )
          painter.drawRect( x,y,w,h-2)    
          boxtext(painter,x,y,w,h,text)
        else:
          painter.fillRect( x,y,w,h*1/8, QBrush(QColor(100,255,100)) )
          painter.drawRect( x,y,w,h*1/8)    
          painter.drawRect( x,y+h*1/8,w,h*7/8)    
          boxtext(painter,x,y,w,h/8,text)
          self.body.draw(painter,x+2,y+h*1/8,w-4,h*7/8)
       
        
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        liste.append(("  "*level+"def "+self.name+"("+self.text+"):",self))
        self.body.make_Pycode(level+1,liste)

###

class Klasse(StdKlasse):
    """
    Eine Klasse, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 

        text2=QCanvasText("Class "+self.name+"("+self.text+")",canvas)
        text2.move(x+w/8,y+2)
        text2.setZ(10)
        items.append(text2)

        pen=QPen(QColor(0,200,200))
        brush=QBrush(QColor(0,200,200))

        if self.is_hidden():
          items.append(QCanvasRectangle ( x,y,w,h-2, canvas ) )
          items[-1].setPen(pen)
          items[-1].setBrush(brush)
        else:
          items.append(QCanvasRectangle ( x,y,w,20, canvas ) )
          items[-1].setPen(pen)
          items[-1].setBrush(brush)

          items.append(QCanvasRectangle ( x,y+20,w,h-20, canvas ) )    
          items[-1].setPen(pen)                 
          items.extend(self.body.out(canvas,x+2,y+22,w-4,h-24))

        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        text="Class "+self.name+"("+self.text+")"

        painter.drawRect( x,y,w,h )    
        if self.is_hidden():
          painter.fillRect( x,y,w,h-2, QBrush(QColor(0,200,200)) )
          painter.drawRect( x,y,w,h-2)    
          boxtext(painter,x,y,w,h,text)
        else:
          painter.fillRect( x,y,w,h*1/8, QBrush(QColor(0,200,200)) )
          painter.drawRect( x,y,w,h*1/8)    
          painter.drawRect( x,y+h*1/8,w,h*7/8)    
          boxtext(painter,x,y,w,h/8,text)
          self.body.draw(painter,x+2,y+h*1/8,w-4,h*7/8)
        
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        if self.text!="":
          liste.append(("class "+self.name+"("+self.text+"):",self))
        else:
          liste.append(("class "+self.name+":",self))
        self.body.make_Pycode(level+1,liste)
                 
###

class Block(StdBlock):
    """
    Ein StdBlock, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[] 

        if w>200:
          offset=12
        else:
          offset=6  

        items.append(QCanvasRectangle ( x,y,offset,h, canvas ) )   
        brush=QBrush(QColor(250,250,0))                
        items[-1].setBrush(brush)


        # DarstellungsGre der Subblcke ber size()/totalsize ermitteln.
        totalsize=0
        for i in self.statements:
          totalsize+=i.size()       
        l=len(self.statements)
        for i in self.statements:
          items.extend(i.out(canvas,x+offset,y,w-offset,h*i.size()/totalsize))
          y=y+h*i.size()/totalsize
        return items 

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        offset=w/32
        painter.fillRect( x,y,offset,h, QBrush(QColor(250,250,0)) )
        painter.drawRect( x,y,offset,h )

        # DarstellungsGre der Subblcke ber size()/totalsize ermitteln.
        totalsize=0
        for i in self.statements:
          totalsize+=i.size()       
        l=len(self.statements)
        for i in self.statements:
          i.draw(painter,x+offset,y,w-offset,h*i.size()/totalsize)
          y=y+h*i.size()/totalsize

    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        for s in self.statements:
           s.make_Pycode(level,liste)

###

class Loop(StdLoop):
    """
    Eine StdLoop, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """

    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))

        my=min(h/4,24)

        items=[]
        text=QCanvasText("while "+self.condition,canvas)
        text.move(x+w/8,y+1)
        items.append(text)
        items.append(QCanvasRectangle ( x+w/16,y+my,w*15/16,h-my, canvas ) )    
        try:
          items.extend(self.body.out(canvas,x+w/16,y+my,w*15/16,h-my ))
        except:
          pass
        return items        
        
    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        my=h/4       
        boxtext(painter,x,y,w,my,"while "+self.condition)
        painter.drawRect( x+w/16,y+my,w*15/16,h-my )    
        try:
          self.body.draw(painter,x+w/16,y+my,w*15/16,h-my)
        except:
          pass
       
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        liste.append(("  "*level + "while " + self.condition + ":",self))
        self.body.make_Pycode(level+1,liste)

###

class Forloop(StdForloop):
    """
    Eine StdForLoop, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """

    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        items=[]
        text=QCanvasText("for "+self.iterator+" in "+self.condition,canvas)
        text.move(x+4,y+2)
        items.append(text)
        items.append(QCanvasRectangle ( x+w/16,y+h/4,w*15/16,h*3/4, canvas ) )    
        try:
          items.extend(self.body.out(canvas,x+w/16,y+h/4,w*15/16,h*3/4 ))
        except:
          pass
        return items
        
    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        my=h/4       
        boxtext(painter,x,y,w,my,"for "+self.iterator+" in "+self.condition)
        painter.drawRect( x+w/16,y+my,w*15/16,h-my )    
        try:
          self.body.draw(painter,x+w/16,y+my,w*15/16,h-my)
        except:
          pass               
        
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        r="  "*level + "for " + self.iterator + " in "+ self.condition + ":"
        liste.append((r,self))
        self.body.make_Pycode(level+1,liste)

###

class Ifthen(StdIfthen):
    """
    StdIfThen, erweitert um canvas-Ausgabe und Codeerstellung.
    Beispiel fr direkte Vererbung.
    """
    def out(self,canvas,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))
        font=QFont()
        font.setPointSize(max(min(h/16,12),5))
        items=[]  
        iftext=QCanvasText("if "+self.condition,canvas)
        iftext.move(x+w/4,y+1)
        iftext.setFont(font)
        
        thentext=QCanvasText("then",canvas)
        thentext.move(x+w/16,y+h/8)
        thentext.setFont(font)
    
        elsetext=QCanvasText("else",canvas)
        elsetext.move(x+w*7/8,y+h/8)    
        elsetext.setFont(font)

        if self.bodytrue.size()>self.bodyfalse.size()+4:
          wl=w*3/4
        elif self.bodytrue.size()<self.bodyfalse.size()-4:
          wl=w/4
        else:    
          wl=w/2
        wr=w-wl
        mx=x+wl
    
        items.append(QCanvasRectangle ( x,y,w,h, canvas) )
        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( x,y,mx,y+h/4)
    
        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( mx,y+h/4,x+w,y+1)
    
        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( mx,y+h/4,mx,y+h)
    
        items.append(QCanvasLine ( canvas ))
        items[-1].setPoints( x+1,y+h/4,x+w,y+h/4)

        items.append(iftext)
        items.append(thentext)
        items.append(elsetext)
        try:
          items.extend(self.bodytrue.out(canvas,x,y+h/4,wl,h*3/4))
          items.extend(self.bodyfalse.out(canvas,mx,y+h/4,wr,h*3/4))
        except:
          pass
        return items

    def draw(self,painter,x,y,w,h):
        "Ausgeben des Statements auf dem canvas an Position x,y,w,h"
        self.setpos((x,y,w,h))

        if self.bodytrue.size()>self.bodyfalse.size()+4:
          wl=w*3/4
        elif self.bodytrue.size()<self.bodyfalse.size()-4:
          wl=w/4
        else:    
          wl=w/2
        wr=w-wl
        mx=x+wl
        my=h/4

        painter.drawRect( x,y,w,my)
        painter.drawLine( x,y,mx,y+my)
        painter.drawLine( mx,y+my, x+w,y)
        boxtext(painter,x+w/4,y,w/2,my,"if "+self.condition,"center")
        boxtext(painter,x,y,wl,my,"then")
        boxtext(painter,mx,y,w-wl,my,"else","right")
       
        try:
          self.bodytrue.draw(painter,x,y+my,wl,h-my)
          self.bodyfalse.draw(painter,mx,y+my,wr,h-my)
        except:
          pass       
   
    def make_Pycode(self,level,liste):
        """
        Erzeuge Python-Code, und schreibe den Code in Liste.
        level=Schactelungstiefe fr Leerzeichen bei Code-Einrckung.
        liste=Liste von (Erzeugter Code,self)
        """
        liste.append(("  "*level + "if " + self.condition + ":",self))
        self.bodytrue.make_Pycode(level+1,liste)
        liste.append(("  "*level + "else:",self))
        self.bodyfalse.make_Pycode(level+1,liste)

#

def main():
  "Test-Code"
  print "MAIN"
  
  print "Statements anlegen"
  s1 = Command()
  s1.setText("a=1")

  app=QApplication(sys.argv)
  canvas=QCanvas(320,256) 
  print s1.out(canvas,0,0,256,256)
  
  print "-----"
  
  s2 = Command()
  s2.setText("b=2")
  b=Block()
  b.appendStmt(s1)
  b.appendStmt(s2)
  l=Loop()
  l.setCond("a>b")
  l.setBody(b)
  
  print "Statement-Objekte:"


  print l.out(canvas,0,0,256,256)
  print "------"
  testliste=[]
  print l.make_Pycode(0,testliste)
  print "ENDE."
  print testliste


if __name__=="__main__":
  main()
  